/* R-Dev-Ino Speech Controller
   - I2C and Bit-Bang Serial communication interfaces
   - accepts voice commands from the VRbot chip
   - uses Webbot's Speech Syntesizer code
   - uses PWM signal and an opamp for speech output
   - uses a serial LCD
   - uses a piezo speaker
   - uses both I2C master and slave mode
*/ 

// R-Dev-Ino board pinout:
//
// Ext  RDI Funct Arduino  ATmega168      Arduino Funct RDI External
//                       +-----\/----+
//          Reset       1| PC6   PC5 |28  D19 A5  SCL
//          Rx    D0    2| PD0   PC4 |27  D18 A4  SDA
//          Tx    D1    3| PD1   PC3 |26  D17 A3        
//     VRrx Int0  D2    4| PD2   PC2 |25  D16 A2        
//     VRtx Int1  D3    5| PD3   PC1 |24  D15 A1        
//                D4    6| PD4   PC0 |23  D14 A0        
//                      7| VCC   GND |22  
//                      8| GND  AREF |21  
//          Xtal        9| PB6  AVCC |20  
//          Xtal       10| PB7   PB5 |19  D13      SCK  LED
//          OC0B  D5   11| PD5   PB4 |18  D12      MISO LCD
//          OC0A  D6   12| PD6   PB3 |17  D11 OC2A MOSI       
//      Speaker   D7   13| PD7   PB2 |16  D10 OC1B      Audio
//                D8   14| PB0   PB1 |15  D 9 OC1A      
//                       +-----------+
//

#include <SoftwareSerial.h>
#include <Wire.h>
#include <Speaker.h>
#include <TTS.h>
#include "protocol.h"

// I2C addresses
#define Compass         0x21
#define Speech          50>>1
#define MotorController 60>>1
#define ServoController 70>>1
#define LedBar          80>>1
#define Mapper          90>>1
#define Brain           100>>1
#define Thermo          208>>1

// I2C commands
#define Forward     1 //MotorController
#define Reverse     2 //MotorController
#define Left        3 //MotorController/ServoController
#define Right       4 //MotorController/ServoController
#define StopAll     5 //MotorController
#define GoFwd       6 //MotorController
#define GoBwd       7 //MotorController
#define Err         8 //MotorController

#define Up          1 //ServoController
#define Down        2 //ServoController

#define Swaist     11 //ServoController
#define SleftSh    12 //ServoController
#define SrightSh   13 //ServoController
#define SleftArm   14 //ServoController
#define SrightArm  15 //ServoController
#define Sscan      16 //ServoController
#define SleftHand  17 //ServoController
#define SrightHand 18 //ServoController
#define SpanHead   19 //ServoController
#define StiltHead  20 //ServoController
#define StorePos   21 //ServoController
#define PickUpSO   22 //ServoController
#define ReleaseSO  23 //ServoController
#define Wave       24 //ServoController
#define PickUpBO   25 //ServoController
#define ReleaseBO  26 //ServoController

#define LCD1       31 //Speech
#define LCD2       32 //Speech
#define Heading    33 //Speech
#define Node       34 //Speech
#define Room       35 //Speech
#define Ping       36 //Speech
#define Sharp      37 //Speech
#define Angle      38 //Speech
#define Compass    39 //Speech
#define Temp       40 //Speech
#define Cluster    41 //Speech
#define FoundFire  42 //Speech
#define NoFire     43 //Speech

#define Scan       61 //Mapper
#define Fire       62 //Mapper
#define Fetch      63 //Mapper
#define Dance      64 //Mapper

#define Done       99 //Mapper

// LCD pins
#define txPin 12         // digital pin 12
#define rxPin 13         // digital pin 13

// Media pins
#define speakerOut 7    // digital pin 7
#define difOut    10    // digital pin 10
#define ledPin    13    // digital pin 13                          

// Software serial
SoftwareSerial LcdSerial =  SoftwareSerial(rxPin, txPin);

// Speaker
Speaker speaker = Speaker(speakerOut); 
Speaker dif = Speaker(difOut); 

// Text to speech
TTS speech;

// I2C
byte I2C[5]={0,0,0,0,0}; 
#define Address   0
#define Command   1
#define Value1    2
#define Value2    3

uint8_t _receivePin;
uint8_t _transmitPin;
long _baudRate;
int _bitPeriod;
int cmd=0;
boolean state=0;
boolean fire=0;
char text [50];

void setup() { 
  //software serial for LCD
  //pinMode(rxPin, INPUT);
  pinMode(txPin, OUTPUT);
  LcdSerial.begin(9600);
  delay(500);
  Splash();
    // PC UART Init
  //pinMode(0, INPUT);         // sets the digital pin as output
  //pinMode(1, OUTPUT);        // sets the digital pin as output
    // VRbot UART Init
  pinMode(2, INPUT);         // sets the digital pin as output
  pinMode(3, OUTPUT);        // sets the digital pin as output 
  //media
  pinMode(ledPin, OUTPUT); 
  pinMode(speakerOut, OUTPUT);  
  pinMode(difOut, OUTPUT);  
    LCDclear();
    LCDposition(1, 1);
    LcdSerial.print("System ready...");
  delay(100);
      speech.setPitch(3);
      strcpy(text, " All  systems  rready. ");
      speech.say(text); 
  delay(1500);
    LCDclear();
  VRcheck();
  //Two wire
  Wire.begin(Speech); // join i2c bus (address optional for master)
  Wire.onReceive(I2CreceiveEvent); // register event
}

//================================================================
// Main Loop
//================================================================
void loop(){
    SD_Recognition();
    //Test_Speech();
    //delay(1000);          // delay a second
    //Process_commands();
    //Find_Hot_Object();
    //delay(100);
}  
//================================================================
void Process_commands() {
    switch (I2C[Command]) {
      case LCD1:
        LCDposition(1, 1);
        SwitchObject();
        I2C[Command]=0;
        break;
      case LCD2:
        LCDposition(2, 1);
        SwitchObject();
        I2C[Command]=0;
        break;
    }
}

void SwitchObject(){
        switch (I2C[Value1]){
          case Heading:
            LcdSerial.print("Heading: ");
            break;
          case Node:
            LcdSerial.print("Node: ");
            break;
          case Ping:
            LcdSerial.print("Ping: ");
            break;
          case Sharp:
            LcdSerial.print("Sharp: ");
            break;
          case Angle:
            LcdSerial.print("Angle: ");
            break;
          case Compass:
            LcdSerial.print("Compass: ");
            break;
          case Temp:
            LcdSerial.print("Temp *C: ");
            break;
          case Cluster:
            LcdSerial.print("Cluster: ");
            break;
           case FoundFire:
            LcdSerial.print("Fire!!!  ");
            delay(100);
            strcpy(text, "Fire, Fire, Fire!");
            speech.say(text); 
            break;
          case NoFire:
            LcdSerial.print("No Fire: ");
            delay(100);
            strcpy(text, "There is no Fire!");
            speech.say(text);
            fire=0; 
            break;
       }
    delay(100);
    LcdSerial.print(I2C[Value2], DEC);
    LcdSerial.print("   ");
}

void VRcheck()
{
  /*if (analogRead(1)<10)
  {
    speaker.Beep();
    LCDposition(1, 1);
    LcdSerial.print("VRbot");
    LCDposition(2, 1);
    LcdSerial.print("Bridge mode");
    while(1)
    {
      int pc2vr = digitalRead(0);
      digitalWrite(13, pc2vr);
    
      int vr2pc = digitalRead(12);
      digitalWrite(1, vr2pc);
    }
  }*/
  
  VRbot_setup();
  
  if (!VRbot_Detect())
  {  
    LCDclear();
    LCDposition(1, 1);
    LcdSerial.print("VRbot NA");
  }else{
    LCDclear();
    LCDposition(1, 1);
    LcdSerial.print("VRbot detected");
    LCDposition(2, 1);
    LcdSerial.print("Lang: English");
    VRbot_SetLanguage(0);  
    delay(1500);
  }   
}

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 void LCDposition (byte line, byte column) {
  LcdSerial.print(line, BYTE);       // LCD line 
  LcdSerial.print(column, BYTE);     // LCD column 
}

void LCDclear () {
  LcdSerial.print(5, BYTE);          // select LCD command mode 
  LcdSerial.print(1, BYTE);          // clear screen
}

void Splash(){
  LCDclear();
  LCDposition(1, 1);
  LcdSerial.print("Mini Eric");
  LCDposition(2, 5);
  LcdSerial.print("robot");
  delay(500);    //wait half a second
  //LCDclear();
}
  
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void I2C_transmit(byte Addr, byte Cmnd, byte Val1, byte Val2)
{
  Wire.beginTransmission(Addr); // transmit to slave device
  Wire.send(Val2);         // sends one byte  
  Wire.send(Val1);         // sends one byte  
  Wire.send(Cmnd);        // sends one byte
  Wire.endTransmission();    // stop transmitting
}


void Test_Speech() {
 // speak text
 strcpy(text, "Hello... My name is Erik. What is your name?");
 speech.say(text); 
}

int I2C_read(int addr, int reg){    // Function to receive one byte of data from slave
  Wire.beginTransmission(addr);       // Begin communication with slave
  Wire.send(reg);                     // Send register to slave
  Wire.endTransmission();
  Wire.requestFrom(addr, 1);          // Request 1 byte
  while(Wire.available() < 1);        // Wait for byte to arrive
  int data = Wire.receive();          // Get byte
  return(data);                       // return byte
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void I2CreceiveEvent(int i)
{
    //Serial.println("I2C communication received:");
    while(0 < Wire.available()) // loop through all
    {
      I2C[i] = Wire.receive(); // receive bytes
      //Serial.print(i, DEC);
      //Serial.print("  ");
      //Serial.println(I2C[i], DEC);
      i = i-1;
    }
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

